# Time-stamp: <2008-06-02 13:45:55 Tao Liu>
"""Module Description: Parse the tag file, store parameters, input and
output filenames

@status:  experimental
@version: $Revision$
"""


import ConfigParser
import string

class ConfigMissingException(Exception):
    pass

class Cye5NumberException(Exception):
    pass

class Cye3NumberException(Exception):
    pass

class WrongConfigValueException(Exception):
    pass

def read(tagFile):
    config = {}
    
    cp = ConfigParser.ConfigParser()

    raw_tag_name  = tagFile.rsplit('.',1)[0]
    
    if len(cp.read(tagFile)) == 0:
        raise IOError("%s not found!" % tagFile)
    
    for sec in cp.sections():
        secName = string.lower(sec)
        for opt in cp.options(sec):
            optName = string.lower(opt)
            config[secName + "." + optName] = string.strip(cp.get(sec, opt))
    
    result = {}
    
    result["tagFile"] = tagFile
    result["sample"] = {}       # for input
    result["output"] = {"dir":"MA2C_Output"} # for output; the
                                             # directory name for
                                             # outpu

    # sample design id
    sampleDesignId = config.setdefault("sample.design_id", None)
    if sampleDesignId == None:
        raise ConfigMissingException("DESIGN_ID in SAMPLE section is missing!")
    
    result["sample"]["designId"] = sampleDesignId
    result["output"]["tpmap"] = "MA2C_Output/MA2C_%s.tpmap" % sampleDesignId # tpmap
                                                                             # filename

    # sample ndf file
    sampleNdfFile = config.setdefault("sample.ndf_file", None)
    if sampleNdfFile == None:
        raise ConfigMissingException("NDF_FILE in SAMPLE section is missing!")
    
    result["sample"]["ndfFile"] = sampleNdfFile
    
    # sample pos file
    samplePosFile = config.setdefault("sample.pos_file", None)
    if samplePosFile != None:
        result["sample"]["posFile"] = samplePosFile
    
    # sample chips, cye5Files & cye3Files  
    chipIds = config.setdefault("sample.chip_id", None)
    if chipIds == None:
        raise ConfigMissingException("CHIP_ID in SAMPLE section is missing!")
    
    chipIdList = chipIds.split()

    result["output"]["FDRtable"] = "MA2C_Output/"+raw_tag_name+"_FDRtable.txt"
    result["output"]["PeakXLS"] = "MA2C_Output/"+raw_tag_name+"_peaks.xls"
    result["output"]["NegPeakXLS"] = "MA2C_Output/"+raw_tag_name+"_negative_peaks.xls"
    result["output"]["PeakBED"] = "MA2C_Output/"+raw_tag_name+"_peaks.bed"
    result["output"]["wiggle"] = "MA2C_Output/"+raw_tag_name+"_MA2Cscore.wig.gz"

    
    cye5Files = config.setdefault("sample.cye5_file", None)
    if cye5Files == None:
        raise ConfigMissingException("CYE5_FILE in SAMPLE section is missing!")
    
    cye5FileList = cye5Files.split()
    
    cye3Files = config.setdefault("sample.cye3_file", None)
    if cye3Files == None:
        raise ConfigMissingException("CYE3_FILE in SAMPLE section is missing!")
    
    cye3FileList = cye3Files.split()
    
    if (len(chipIdList) != len(cye5FileList)):
        raise Cye5NumberException("The number of CYE5_FILE in SAMPLE section does not match the number of CHIP_ID!")
    
    if (len(chipIdList) != len(cye3FileList)):
        raise Cye3NumberException("The number of CYE3_FILE in SAMPLE section does not match the number of CHIP_ID!")
    
    # put chips in the result
    result["sample"]["chips"] = {}
    result["output"]["chips"] = {}
    
    for i in xrange(len(chipIdList)):
        chipId = chipIdList[i]
        result["sample"]["chips"][chipId] = {}
        result["output"]["chips"][chipId] = {}
        result["sample"]["chips"][chipId]["cye5File"] = cye5FileList[i]
        result["sample"]["chips"][chipId]["cye3File"] = cye3FileList[i]
        result["output"]["chips"][chipId]["raw"] = "MA2C_Output/MA2C_%s_raw.txt" % chipId
        result["output"]["chips"][chipId]["normalized"] = "MA2C_Output/MA2C_%s_normalized.txt" % chipId

        
    result["normalization"] = {}
    
    method = config.setdefault("normalization.method", None)
    if method == None:
        raise ConfigMissingException("METHOD in NORMALIZATION section is missing!")
    method = string.lower(method)
    
    if method != "simple" and method != "robust":
        raise WrongConfigValueException("METHOD in NORMALIZATION section must be Simple or Robust!")
    
    result["normalization"]["method"] = method
    
    if (method == "robust"):
        c = config.setdefault("normalization.c", None)
        if c == None:
            raise ConfigMissingException("C in NORMALIZATION section is missing!")
        result["normalization"]["c"] = int(c)
    
    # peak detection
    result["peakDetection"] = {}
    
    method = config.setdefault("peak detection.method", None)
    if method == None:
        raise ConfigMissingException("METHOD in PEAK DETECTION section is missing!")
    
    result["peakDetection"]["method"] = string.lower(method)
    
    threshold = config.setdefault("peak detection.threshold", None)
    if threshold == None:
        raise ConfigMissingException("THRESHOLD in PEAK DETECTION section is missing!")
    
    result["peakDetection"]["threshold"] = float(threshold)
    
    bandwidth = config.setdefault("peak detection.bandwidth", None)
    if bandwidth == None:
        raise ConfigMissingException("BANDWIDTH in PEAK DETECTION section is missing!")
    
    result["peakDetection"]["bandwidth"] = int(bandwidth)
    
    minProbes = config.setdefault("peak detection.min_probes", None)
    if minProbes == None:
        raise ConfigMissingException("MIN_PROBES in PEAK DETECTION section is missing!")
    
    result["peakDetection"]["minProbes"] = int(minProbes)
    
    maxGap = config.setdefault("peak detection.max_gap", None)
    if maxGap == None:
        raise ConfigMissingException("MAX_GAP in PEAK DETECTION section is missing!")
    
    result["peakDetection"]["maxGap"] = int(maxGap)
    
    return result
